/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
+++
+++     (C) Paragon Software, 1999 - 2001.
+++     Author: Isaev Andrey.
+++     File:   PiLocInterface.H
+++
+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
+++
+++     This is PUBLIC source. No warranty is supplied. Use at your own
+++     risk.
+++
+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
+++
+++	Here is a definitions of <PiLoc Common Interface> for  comunication
+++	with PiLoc Palm localization software internal functions
+++
+++	Current interface version:	109
+++
+++     Modification history:
+++
+++     ver 109 - 23/1/01
+++                   + PiLocCurrentLanguageNameStringTranslated
+++     ver 108 - 23/1/01
+++                   * changes in PiLocGetPortableKeyboardName
+++                     and PiLocGetOnscreenKeyboardName
+++     ver 107 - 20/12/00
+++                   + added support for logo bitmap display
+++	
+++     ver 106 - 08/12/00
+++                   + added support for several onscreen keyboards
+++                   + new version of HWR keyboard support with 
+++             different layouts
+++
+++     ver 105 - 13/10/00 Interface translation support
+++
+++     ver 104 - 23/09/00 serial numbers support,
+++               getting info if this is a demo version of 
+++               language database
+++
+++		ver 103 - 31/7/00 bulk addition: codepages, redraw method
+++
+++		ver 102 - added possibility to change graffity layout
+++				  thru PiLoc API
+++     ver 101	- added support for demo version checking
+++     ver 100 - first version
+++
++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/


#ifndef	_PILOC_INTERFACE_H__
#define _PILOC_INTERFACE_H__


//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//
// Main constants
//
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

#define PiLocCreatorID                                  'RusP'
#define PiLocApplicationType                            'appl'
#define PiLocDataSignature                              'PiLo'

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

#define PiLocOnFeatureNumber                            1
#define PiLocDataPointerFeatureNumber                   2

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

#define CurrentInterfaceVersion                         109
		// This version number represent set of features supported by this
		// version of interface (see medification history). Gratear number 
		// means newer version, thus more fetures is supported.
		// Application should check this number to ensure PiLoc is new 
		// egnouth to support demanded features.
#define MinimalInterfaceVersion                         100
		// This number shows minimal compatible version number.
		// this number will grow only if some features became
		// unsupported or change its behavior violently.
		
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

#define LanguageNameLength                              16
#define ResVersionNameLength                            16
#define KeyboardLayoutNameLength                        16  // max length of buffer
                                                            // to contain length of 
                                                            // layout of keyboard both
                                                            // onscreen and portable


//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//
// Language identifyers definition
//
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

#define  langDefault	    0

#define  langRussian        'russ'
#define  langGreek          'gree'
#define  langPolish         'poli'
#define  langTurkish        'turk'
#define  langHungarian      'hung'
#define  langBulgarian      'bulg'
#define  langCroatian       'croa'
#define  langSlovenian      'slvn'
#define  langSlovak         'slvk'
#define  langCzech          'czec'
#define  langRomanian       'roma'
#define  langUkrainian      'ukra'
#define  langIcelandic      'icel'
#define  langEstonian       'esto'

#define  langHebrew		    'hebr'		// Right-to-left languages 
#define  langArabic         'arab'      // are unsupported yet


//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//
// feature set definition
//
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

// List of features can be supported by PiLoc system
enum	enumPiLocSytemFeatures
{
    featureTranslatedLanguageName = 96,
    featureLogoBitmapDisplay = 97,           
    featureSerialNumber = 98,           
	featureDemoVersion=99,				// Always turned on when PiLoc is Demo,
										// else not supported
		
	featurePrintingDatesInHebrewCalendar=100, // not implemented in current version
	featureDaySelectorInHebrewCalendar, // of interface
	featureRighToLeftCharsRearranging,
	
	
	featureSecondGraffitiLayout =200,	// This feature is supported
										// only if current language supports
										// two graffiti layouts (Russian, Greek,
										// Hebrew).
	featureGrfNumOfDictionaries =205,
	featureGrfCurrentDictionary,
	featureGrfDictinaryName,
										
	featureCodePagesNumber		=210,
	featureCurrentCodePage,	
	featurePopUpCodePageList,
	featureCodePageName,
	featureCodePageEncodeTable,
	featureCodePageDecodeTable,
	featureScreenRedrawMethod,
	
	featurePortableKeyboardLayout = 220,
	featurePortableKeyboardLayoutsNumber,
	featurePortableKeyboardLayoutName,
	
	featureInterfaceTranslation =230,
	featureInterfaceResourcesLanguage,
	featureInterfaceResourcesVersion,
	featureInterfaceUpdateLauncherDB,
	featureInterfaceAppCategories,
	
	featureOnscreenKeyboardLayout = 240,
	featureOnscreenKeyboardLayoutsNumber,
	featureOnscreenKeyboardLayoutName

};



//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//
// definition of prototypes of PiLoc interface methods
//
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

// This function returns state of particular feature
typedef UInt32 (*GetPiLocFeatureStatePtr) (enum enumPiLocSytemFeatures, ...);
	
// This function sets (if possible) features and returns new state
typedef UInt32 (*SetPiLocFeatureStatePtr) (enum enumPiLocSytemFeatures, UInt16 param1, ...);


//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//
// Structure of PiLoc globals data
//
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

typedef 	struct
{
	UInt32		signature	;				// 'PiLo'	signature
	UInt16		intefaceCurrentVersion;		// current version of inteface supperted by this PiLoc
	UInt16		intefaceMinimalVersion;		// minimal compatible version of PiLoc interface
	UInt16		pilocVersion;				// Version of PiLoc multiplayed by 100
	UInt32      langCode;                   // code of installed system language
	
	GetPiLocFeatureStatePtr	GetPiLocFeatureState;	// Methods of this structure
	SetPiLocFeatureStatePtr	SetPiLocFeatureState;
	
	Char		langName[LanguageNameLength];		// Name of current language
	Char		resVersion[ResVersionNameLength];	// information about resources version
}
PiLocGlobalDataHeaderType;
typedef PiLocGlobalDataHeaderType* 	PiLocGlobalDataHeaderPtr;


//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//
// Functions to work with PiLoc globals
//
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

//
// Retreives PiLoc globals pointer
//
// use like: void PiLocGetGlobalsPointer(PiLocGlobalDataHeaderPtr *);
//
#define PiLocGetGlobalsPointer(GlobalsPtrPtr)	(*GlobalsPtrPtr=NULL,FtrGet(PiLocCreatorID,PiLocDataPointerFeatureNumber,(UInt32 *)GlobalsPtrPtr))

//
// validates globals pointer 
//
#define PiLocIsValidGlobalsPointer(Globals)		(GlobalsPtr && (GlobalsPtr == PiLocDataSignature))

//
// Checks for current language
//
#define PiLocIsLanguageSet(GlobalsPtr,LangCode)	(GlobalsPtr->langCode==LangCode)

//
// Checks compatibility of this API version with one used with Started PiLoc
//
#define PiLocIsAPIVersionCompatible(GlobalsPtr,APIVersionNum)	(GlobalsPtr->intefaceCurrentVersion >= APIVersionNum && GlobalsPtr->intefaceMinimalVersion <= APIVersionNum)

//
// version defines to use with this function
#define PILOC_API_VER_1_02 102					// use it if only graffiti layout switching
												// support is used by application
									
#define PILOC_API_VER_1_03 CurrentInterfaceVersion	// Use if other functions (f.e. codepage
													// support) are required

#define PILOC_API_VER_LATEST CurrentInterfaceVersion // Be carefull using it. Your 
													// software may loose compatibility
													// with older PiLoc versions
													
//
// All-in-one check
//													
#define PiLocCheckCompatibility(Globals,LangCode,ApiVersion)  (PiLocIsValidGlobalsPointer(Globals) && PiLocIsLanguageSet(GlobalsPtr,LangCode) && PiLocIsAPIVersionCompatible(GlobalsPtr,APIVersionNum))


//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++


#define PiLocCurrentLanguageNameString(Globals)                   (Globals->langName)
#define PiLocCurrentResourcesVersionString(Globals)               (Globals->resVersion)
//  This functions are available since 109 version
#define PiLocCurrentLanguageNameStringTranslated(Globals,BufPtr)  (Globals->GetPiLocFeatureState(featureTranslatedLanguageName,(Char*)(BufPtr)))

                         
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//
// PiLoc calls definition
//
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

// This is returned if called function is unsupported
#define pilocFeatureIsUnsupported		((UInt32)-1)

// This used to enable/disable some functions
#define PILOC_FEATURE_ON                     1
#define PILOC_FEATURE_OFF                    0


//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// ================================================
//  This functions are available since 102 version
// ================================================
#define	PiLocGetGraffitiLayout(GlobalsPtr)			(GlobalsPtr->GetPiLocFeatureState(featureSecondGraffitiLayout))
			// retuns current graffiti layout (basic or second)
			// or pilocFeatureIsUnsupported if current language
			// has no second graffitilayout.
#define	PiLocSetGraffitiLayout(GlobalsPtr,Layout)	(GlobalsPtr->SetPiLocFeatureState(featureSecondGraffitiLayout,Layout))
			// sets or switches current graffiti layout, and
			// returns new layout or pilocFeatureIsUnsupported
			// if current language has no second graffitilayout.

//
// Layouts for PiLocGetGraffitiLayout/PiLocSetGraffitiLayout
//
#define pilocBasicGraffitiLayout			0
#define pilocSecondGraffitiLayout			1
#define pilocSwitchGraffitiLayout			100	// this used only for setting layout


//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// ================================================
//  This functions are available since 103 version
// ================================================
#define	PiLocGetNumberOfGraffitiDictionaries(GlobalsPtr)        (GlobalsPtr->GetPiLocFeatureState(featureGrfNumOfDictionaries))
			// Returns number of Graffiti dictionaries in
			// current language database
#define	PiLocGetCurrentGraffitiDictionary(GlobalsPtr)           (GlobalsPtr->GetPiLocFeatureState(featureGrfCurrentDictionary))
			// This function returns index of current Graffiti dictionary
#define	PiLocSetCurrentGraffitiDictionary(GlobalsPtr,DictIndex) (GlobalsPtr->SetPiLocFeatureState(featureGrfCurrentDictionary,DictIndex))
			// This function sets current system-wide Graffiti dictionary
			// it also changes current graffiti help
#define	PiLocGetGraffitiDictionaryName(GlobalsPtr,DictIndex)    (Char*)(GlobalsPtr->SetPiLocFeatureState(featureGrfDictinaryName,DictIndex))
			// Returns pointer to name of Graffiti dictionary specified by index.
			// Returns NULL if error

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
#define	PiLocGetNumberOfSupportedCodePages(GlobalsPtr)    (GlobalsPtr->GetPiLocFeatureState(featureCodePagesNumber))
			// Returns number of codepages supported by
			// current language
#define	PiLocGetCurrentCodePage(GlobalsPtr)               (GlobalsPtr->GetPiLocFeatureState(featureCurrentCodePage))
			// This function returns index of current codepage
#define	PiLocSetCurrentCodePage(GlobalsPtr,CodePageIndex) (GlobalsPtr->SetPiLocFeatureState(featureCurrentCodePage,CodePageIndex))
			// This function sets current system-wide codepage
			// it does not any redrawing, but everithing that will
			// be printed after this call will use new codepage.
			// Returns index of current codepage
#define	PiLocPopUpCodePagesList(GlobalsPtr)               (GlobalsPtr->SetPiLocFeatureState(featurePopUpCodePageList,0))
			// This function brings up list of available
			// codepages in the right bottom of screen, traces
			// user selection, changes current codepage if 
			// necessary and redraws the screen.
			// Note, if current method of redrawing is restarting
			// of application, current pallication will be restarted
			// Returns index of current codepage
#define	PiLocGetCodePageName(GlobalsPtr,CodePageIndex)    (Char*)(GlobalsPtr->SetPiLocFeatureState(featureCodePageName,CodePageIndex))
			// Returns pointer to name of codepage specified by index.
			// Return NULL if current language does not support
			// codepages or index is out of range
#define	PiLocGetCurrentCodeEncodeTable(GlobalsPtr)        (UInt8*)(GlobalsPtr->GetPiLocFeatureState(featureCodePageEncodeTable))
			// Returns pointer to conversion table from
			// basic codepage to current one or null if no 
			// conversion needed or no additional codepages
			// are in current language.
#define	PiLocGetCurrentCodeDecodeTable(GlobalsPtr)        (UInt8*)(GlobalsPtr->GetPiLocFeatureState(featureCodePageDecodeTable))
			// Table for decoding from current codepage
			// to default.
#define	PiLocGetCodeEncodeTableByIndex(GlobalsPtr,CodePageIndex) (UInt8*)(GlobalsPtr->SetPiLocFeatureState(featureCodePageEncodeTable,CodePageIndex))
#define	PiLocGetCodeDecodeTableByIndex(GlobalsPtr,CodePageIndex) (UInt8*)(GlobalsPtr->SetPiLocFeatureState(featureCodePageDecodeTable,CodePageIndex))
			// Same, but gets table using index of codepage


//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
#define	PiLocGetCurrentScreenRedrawMethod(GlobalsPtr)        (GlobalsPtr->GetPiLocFeatureState(featureScreenRedrawMethod))
#define	PiLocSetCurrentScreenRedrawMethod(GlobalsPtr,Method) (GlobalsPtr->SetPiLocFeatureState(featureScreenRedrawMethod,Method))
			// Get/Set current method of screen redraw used
			// when used changes current codepage
//
// Methods of screen redrawing
//
#define pilocScrRedrawMethod1			0	// Safe but unconvinient method
											// with restarting of current
											// application
#define pilocScrRedrawMethod2			1	// More convinient but risky
											// method with redrawing 
											// of all visible forms

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// ================================================
//  This functions are available since 104 version
// ================================================
#define	PiLocGetLangDatabaseSN(GlobalsPtr,BuffPtr)                (GlobalsPtr->GetPiLocFeatureState(featureSerialNumber,(char *)BuffPtr))
            // Stores into destination buffer of size _MIN_SN_BUFF_SIZE_
            // serial number string of currently active language database.
            // Does nothing if no database is selected
            // Returns TRUE(1) if OK or FALSE(0) if no current language database
            // is selected

#define _MIN_SN_BUFF_SIZE_    23
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
#define	PiLocIsDemoVersion(GlobalsPtr)                            (GlobalsPtr->GetPiLocFeatureState(featureDemoVersion))
            // Checks if currently used database is 
            // demo version of language database
            // Returns: TRUE(1) if DEMO or FALSE(0) if registred commertial
            // If no language database is currently active returns FALSE.


//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// ================================================
//  This functions are available since 105 version
// ================================================
#define	PiLocGetInterfaceResourcesLanguage(GlobalsPtr)            (Char*)(GlobalsPtr->GetPiLocFeatureState(featureInterfaceResourcesLanguage))
            // Returns pointer to string containing information
            // about language of resources database
            // Returns pilocNoInterfaceTranslation If no Interface
            // Translation feature is supported and/or activated

#define	PiLocGetInterfaceResourcesVersion(GlobalsPtr)             (Char*)(GlobalsPtr->GetPiLocFeatureState(featureInterfaceResourcesVersion))
            // Returns pointer to string containing information
            // about VERSION of resources database
            // Returns pilocNoInterfaceTranslation If no Interface
            // Translation feature is supported and/or activated

#define	PiLocEnableInterfaceTranslation(GlobalsPtr,Enable)        (Char*)(GlobalsPtr->SetPiLocFeatureState(featureInterfaceTranslation,Enable))
            // Enables/disables Interface translation.
            // This feature requires resource database installed
            // on device. Returns pilocFeatureIsUnsupported if feature
            // is unsupported or no resource file found.

#define	PiLocIsInterfaceTranslationEnabled(GlobalsPtr)            (GlobalsPtr->GetPiLocFeatureState(featureInterfaceTranslation))
            // Returns state of interface translation feature
            // or tells that no resource database is installed

#define	PiLocUpdateLauncherDB(GlobalsPtr)                         (GlobalsPtr->SetPiLocFeatureState(featureInterfaceUpdateLauncherDB,0))
            // After changing interface translation language
            // database of App Launcher must be updated ( OS 3.5 only)

#define	PiLocUpdateBuiltInAppCategories(GlobalsPtr)                  (GlobalsPtr->SetPiLocFeatureState(featureInterfaceAppCategories,0))
            // After changing interface translation language
            // Built-in app categories should be updated
            
#define pilocInterfaceTranslationEnabled     PILOC_FEATURE_ON
#define pilocInterfaceTranslationDisabled    PILOC_FEATURE_OFF
#define pilocInterfaceTranslationNoDatabase  pilocFeatureIsUnsupported
            
#define pilocNoInterfaceTranslation   (Char*)pilocFeatureIsUnsupported


//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// ================================================
//  This functions are available since 106 version
// ================================================

#define	PiLocGetNumberOfOnscreenKeyboardLayouts(GlobalsPtr)          (GlobalsPtr->GetPiLocFeatureState(featureOnscreenKeyboardLayoutsNumber))
			// Returns number of Keyboard resources in
			// current language database
#define	PiLocGetCurrentOnscreenKeyboard(GlobalsPtr)                  (GlobalsPtr->GetPiLocFeatureState(featureOnscreenKeyboardLayout))
			// This function returns index of current Keyboard resource
#define	PiLocSetCurrentOnscreenKeyboard(GlobalsPtr,KeybdIndex)       (GlobalsPtr->SetPiLocFeatureState(featureOnscreenKeyboardLayout,KeybdIndex))
			// This function sets current onscreen Keyboard resource
#define	PiLocGetOnscreenKeyboardName(GlobalsPtr,KeybdIndex,BufPtr)          (Char*)(GlobalsPtr->GetPiLocFeatureState(featureOnscreenKeyboardLayoutName,(UInt16)(KeybdIndex),(Char*)(BufPtr)))
			// Stores name of Keyboard resource specified by index.
			// to the buffer given
			// Returns BufPtr or NULL if error

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

#define	PiLocGetNumberOfPortableKeyboardLayouts(GlobalsPtr)          (GlobalsPtr->GetPiLocFeatureState(featurePortableKeyboardLayoutsNumber))
			// Returns number of basic layouts of Portable Keyboard in
			// current language database or pilocFeatureIsUnsupported
			// if no Portable Keyboard resource is currently available
#define	PiLocGetCurrentPortableKeyboardLayout(GlobalsPtr)                  (GlobalsPtr->GetPiLocFeatureState(featurePortableKeyboardLayout))
			// This function returns index of current Portable 
			// Keyboard layout:
			// pilocFeatureIsUnsupported - feature is unavailable
			// 0 - support is off
			// 1 - first layout
			// 2 - second...
#define	PiLocSetCurrentPortableKeyboardLayout(GlobalsPtr,KeybdIndex)       (GlobalsPtr->SetPiLocFeatureState(featurePortableKeyboardLayout,KeybdIndex))
			// This function sets current portable Keyboard layout
			// numeration of lyouts is the same
#define	PiLocGetPortableKeyboardName(GlobalsPtr,KeybdIndex,BufPtr)         (Char*)(GlobalsPtr->GetPiLocFeatureState(featurePortableKeyboardLayoutName,(UInt16)(KeybdIndex),(Char*)(BufPtr)))
			// Stores name of Keyboard resource specified by index to bufer
			// Returns BufPtr or NULL if error


//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// ================================================
//  This functions are available since 107 version
// ================================================

#define	PiLocDisplayLogoBitmap(GlobalsPtr,Seconds)                         (GlobalsPtr->SetPiLocFeatureState(featureLogoBitmapDisplay,Seconds))


//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

#endif //_PILOC_INTERFACE_H__
