#include <PalmOS.h>
#include "callback.h"
#include "RuSMS.h"

static DmOpenRef SmsDB = NULL;
static Char **smsList = NULL;
static UInt16 smsListCnt = 0;
Char *InsertText = NULL;

Boolean OpenSmsList()
{
	Err err;
	if (SmsDB != NULL) return true;
	SmsDB = DmOpenDatabaseByTypeCreator(SavedSmsDBType, APPID,
										 dmModeReadWrite);
	if (SmsDB == NULL) {
		err = DmCreateDatabase (0, "SavedSmsDB", APPID, SavedSmsDBType,
								false);
		if (err) {
			SmsDB = NULL;
			SystemError(err);
			return false;
		}
		SmsDB = DmOpenDatabaseByTypeCreator(SavedSmsDBType, APPID,
											 dmModeReadWrite);
	}
	return true;
}

void CloseSmsList()
{
	if (SmsDB != NULL)
		DmCloseDatabase(SmsDB);
	SmsDB = NULL;
}

void SmsListFree()
{
	UInt16 i;
	if (smsList == NULL) return;
	for (i=0; i<smsListCnt; i++)
		if (smsList[i] != NULL)
			MemPtrFree(smsList[i]);
	smsListCnt = 0;
}

Boolean LoadSmsList(ListPtr lst)
{
	UInt16 recs, i, l, r, recSize;
	MemHandle recH;
	MemPtr recP, p;
	Char *str;
	FontID curfont;
	Int16 sw, sl;
	Boolean fww;
	if ((recs = DmNumRecords(SmsDB)) == 0) {
		SmsListFree();
		LstSetListChoices(lst, smsList, smsListCnt);
		return true;
	}
	smsList = (Char**)MemPtrNew(sizeof(Char*)*recs);
	smsListCnt = recs;
	for (r = 0; r < recs; r++) smsList[r] = NULL;
	curfont = FntSetFont(lst->font);
	for (r = 0; r < recs; r++) {
		if((recH = DmQueryRecord(SmsDB, r)) == NULL) {
			SmsListFree();
			FntSetFont(curfont);
			SystemError(DmGetLastErr());
			CloseSmsList();
			return false;
		}
		recSize = MemHandleSize(recH);
		p = recP = MemHandleLock(recH);
		MemMove(&l, p, sizeof(UInt16)); p += sizeof(UInt16);
		str = (Char*)MemPtrNew(l);
		MemMove(str, p, l); p += l;
		MemPtrUnlock(recP);
		sw = lst->bounds.extent.x;
		for (i=0; i<l; i++)
			if (str[i] == chrLineFeed)
				break;
		str[i] = chrNull;
		sl = i;
		FntCharsInWidth(str, &sw, &sl, &fww);
		if (!fww || (l > i)) {
			smsList[r] = (Char*)MemPtrNew(sl+10);
			StrNCopy(smsList[r], str, sl);
			smsList[r][sl] = chrNull;
			StrCat(smsList[r], "...");
			MemPtrFree(str);
		} else
			smsList[r] = str;
	}
	FntSetFont(curfont);
	LstSetListChoices(lst, smsList, smsListCnt);
	return true;
}

Boolean SaveSmsFormHandleEvent (EventPtr e)
{
    Boolean handled = false;
    FormPtr frm;
    ListPtr lst;
	Int16 l;
	EventType evt;
	UInt16 recSize;
	MemHandle recH;
	MemPtr recP, p;

    CALLBACK_PROLOGUE

	switch (e->eType) {
    case frmOpenEvent:
        frm = FrmGetActiveForm();
		lst = FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, SaveSmsForm_SmsList));
		if (OpenSmsList() && LoadSmsList(lst))
			FrmDrawForm(frm);
		else
			FrmReturnToForm(MainForm);
		CloseSmsList();
        handled = true;
        break;

	case frmCloseEvent:
		SmsListFree();
		FrmReturnToForm(MainForm);
		handled = true;
		break;

    case ctlSelectEvent:
        switch(e->data.ctlSelect.controlID) {
		case SaveSmsForm_DoneButton:
			frm = FrmGetActiveForm();
			lst = FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, SaveSmsForm_SmsList));
			if ((l = LstGetSelection(lst)) == noListSelection) break;
			if (!OpenSmsList()) break;
			if((recH = DmQueryRecord(SmsDB, l)) == NULL) {
				SystemError(DmGetLastErr());
				CloseSmsList();
				SmsListFree();
				break;
			}
			recSize = MemHandleSize(recH);
			p = recP = MemHandleLock(recH);
			MemMove(&l, p, sizeof(UInt16)); p += sizeof(UInt16);
			InsertText = (Char*)MemPtrNew(l);
			MemMove(InsertText, p, l); p += l;
			MemPtrUnlock(recP);
			CloseSmsList();
			SmsListFree();

 			FrmReturnToForm(MainForm);
			evt.eType = evtMainFormInsertTextEvent;
			EvtAddEventToQueue(&evt);
			handled = true;
			break;
		case SaveSmsForm_CancelButton:
			SmsListFree();
 			FrmReturnToForm(MainForm);
			handled = true;
			break;
		case SaveSmsForm_DelButton:
			frm = FrmGetActiveForm();
			lst = FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, SaveSmsForm_SmsList));
			if ((l = LstGetSelection(lst)) == noListSelection) break;
			if (OpenSmsList()) {
				if ((recSize = DmNumRecords(SmsDB)) > 0) {
					if (recSize > l) {
						DmRemoveRecord(SmsDB, l);
						SmsListFree();
						LoadSmsList(lst);
						LstDrawList(lst);
					}
				}
				CloseSmsList();
			}
			handled = true;
			break;
		}
		break;

    default:
        break;
    }

    CALLBACK_EPILOGUE

    return handled;
}

void SaveSms(Char *text)
{
	UInt16 recs, i, recIndex, recSize, off;
	MemHandle recH;
	MemPtr recP;
	if ((text == NULL) || (text[0] == chrNull))
		return;
	if (!OpenSmsList()) return;
	recs = DmNumRecords(SmsDB);
	if (recs >= globalFlags.MaxSavedSms)
		for (i = 0; i < (recs - globalFlags.MaxSavedSms + 1); i++) {
			DmRemoveRecord(SmsDB, 0);
		}
	i = StrLen(text)+1;
	recSize =
		sizeof(UInt16) + /* length */
		i; /* text */
	recIndex = dmMaxRecordIndex;
	if (!(recH = DmNewRecord(SmsDB, &recIndex, recSize))) {
		SystemError(DmGetLastErr());
		CloseSmsList();
		return;
	}
	recP = MemHandleLock(recH);
	off = 0;
	DmWrite(recP, off, &i, sizeof(UInt16));
	off +=  sizeof(UInt16);
	DmWrite(recP, off, text, i);
	off += i;
	MemPtrUnlock(recP);
	DmReleaseRecord(SmsDB, recIndex, true);
	CloseSmsList();
}
