// -*- C++ -*-
// File: SendSms.c
//
// Created: Sat Dec  2 23:24:22 2000
//
// $Id: SendSms.c,v 1.1.1.1 2002/07/17 12:47:33 oskin Exp $
//

#include <PalmOS.h>
#include "callback.h"
#include "RuSMS.h"
#include "cSMS/Serial.h"

#define rcvBufSize 1024

static ProgressPtr prgP;

Boolean rcvDebug;

#define MaxCountLineErr 3

Char *GetStrFromRes(DmResID res)
{
	static Char msg[100];
	MemHandle resH;
	MemPtr resP;

	resH = DmGetResource('tSTR', res);
	resP = MemHandleLock(resH);
	StrNCopy(msg, resP, 100);
	MemPtrUnlock(resP);
	DmReleaseResource(resH);
	return msg;
}

Boolean prgCallback (PrgCallbackDataPtr cbP) {
	Char *p;
	UInt16 l;
	DmResID id;
	
	CALLBACK_PROLOGUE;

	if (cbP->stage == 0)
		id = strOpenModem;
	else
		id = cbP->stage;

	p = GetStrFromRes(id);
	l = cbP->textLen - 1;
	l -= StrLen(p);
	StrNCat(cbP->textP, p, l);
	if (cbP->message != NULL) {
		StrNCat(cbP->textP, "", l);
		StrNCat(cbP->textP, cbP->message, l);
	}
	StrNCat(cbP->textP, "...", l);
	
    CALLBACK_EPILOGUE;

	return true;
}

#define Exit(stat) {\
    SerialClose(portID);\
	PrgStopDialog(prgP, false);\
	return stat;\
}

#define PrgCancelTest {\
    if (PrgUserCancel(prgP)) {\
	PrgUpdateDialog(prgP, 0, strCanceled, NULL, true);\
    Exit(false);\
}}

#define ATerr {\
    Char *s; \
    switch(err) { \
    case errModemError: \
	    s = ""; \
    case errModemErrMsg: \
        s = rcvBuf; \
        ErrMsgRes(0, strModemErr, s); \
	    break; \
    } \
    if(err) Exit(false); \
}

static Int32 saveCMEE, saveCMGF, saveCSMS;
static Boolean Phase2p;
Char rcvBuf[rcvBufSize], PrgTitle[30];
UInt16 portID;

Boolean SendSmsStart()
{
	Err err;

#ifdef DEBUG_SERIAL
	rcvDebug = false;
#endif
	
	StrNCopy(PrgTitle, GetStrFromRes(strSendSMS), 30);
	if (PalmOSVersion >= 0x03203000)
		prgP = PrgStartDialog( PrgTitle, prgCallback, NULL);
	else
		prgP = PrgStartDialogV31( PrgTitle, prgCallback);
	if (prgP == NULL) {
		ErrMsg(memErrNotEnoughSpace, "");
		return false;
	}
#ifdef DEBUG
	return true;
#endif
	PrgUpdateDialog(prgP, 0, strOpenModem, NULL, true);
	if ((portID = SerialOpen(&globalConn)) == 0) {
		PrgStopDialog(prgP, false);
		return false;
	}
	SerialReceiveFlush(portID, globalFlags.SWait);
	SrmSendFlush(portID);//new oleg
	SetWaitTime(globalFlags.Timeout);
	
	PrgUpdateDialog(prgP, 0, strInitModem, NULL, true);
	if (globalFlags.ModemSetup) {
		PrgCancelTest;
		if (SerialSendString(portID, "AT+CMEE?\r")) Exit(false);
		err = SerialReceiveString(portID, rcvBuf, prgP);
		PrgCancelTest;
		ATerr;
		if (StrNCaselessCompare(rcvBuf, "+CMEE:", 6) == 0) {
			saveCMEE = StrAToI(&rcvBuf[5]);
			if ((saveCMEE < 0) || (saveCMEE > 2)) {
				ErrMsgRes(0, strModemCmdErr, rcvBuf);
				Exit(false);
			}
		} else {
			ErrMsgRes(0, strModemCmdErr, rcvBuf);
			Exit(false);
		}
		if (saveCMEE != 2) {
			if (SerialSendString(portID, "AT+CMEE=2\r")) Exit(false);
			err = SerialReceiveString(portID, rcvBuf, prgP);
			PrgCancelTest;
			ATerr;
		}
	}
	 
	if (globalFlags.ModemSetup) {
		PrgCancelTest;
		if (SerialSendString(portID, "AT+CMGF?\r")) Exit(false);
		err = SerialReceiveString(portID, rcvBuf, prgP);
		PrgCancelTest;
		ATerr;
		if (StrNCaselessCompare(rcvBuf, "+CMGF:", 6) == 0) {
			saveCMGF = StrAToI(&rcvBuf[5]);
			if ((saveCMGF < 0) || (saveCMGF > 1)) {
				ErrMsgRes(0, strModemCmdErr, rcvBuf);
				Exit(false);
			}
		} else {
			ErrMsgRes(0, strModemCmdErr, rcvBuf);
			Exit(false);
		}
		if (saveCMEE != 0) {
			if (SerialSendString(portID, "AT+CMGF=0\r")) Exit(false);
			err = SerialReceiveString(portID, rcvBuf, prgP);
			PrgCancelTest;
			ATerr;
		}
	} else {
		if (SerialSendString(portID, "AT+CMGF=0\r")) Exit(false);
		err = SerialReceiveString(portID, rcvBuf, prgP);
		PrgCancelTest;
		ATerr;
	}

	Phase2p = true;
	if (globalFlags.ModemSetup) {
		PrgCancelTest;
		if (SerialSendString(portID, "AT+CSMS?\r")) Exit(false);
		err = SerialReceiveString(portID, rcvBuf, prgP);
		PrgCancelTest;
		ATerr;
		if (StrNCaselessCompare(rcvBuf, "+CSMS:", 6) == 0) {
			saveCSMS = StrAToI(&rcvBuf[5]);
			if ((saveCSMS < 0) || (saveCSMS > 1)) {
				ErrMsgRes(0, strModemCmdErr, rcvBuf);
				Exit(false);
			}
		} else {
			ErrMsgRes(0, strModemCmdErr, rcvBuf);
			Exit(false);
		}
		if (saveCSMS != 1) {
			PrgCancelTest;
			if (SerialSendString(portID, "AT+CSMS=1\r")) Exit(false);
			err = SerialReceiveString(portID, rcvBuf, prgP);
			PrgCancelTest;
			if ((err == errModemError) || (err == errModemErrMsg)) {
				Phase2p = false;
			} else if (err) Exit(false);
		}
		if (Phase2p) {
			PrgCancelTest;
			if (SerialSendString(portID, "AT+CNMA=?\r")) Exit(false);
			err = SerialReceiveString(portID, rcvBuf, prgP);
			PrgCancelTest;
			if ((err == errModemError) || (err == errModemErrMsg)) {
				Phase2p = false;
			} else if (err) Exit(false);
		}
	}
	if (Phase2p)
		return true;
	else {
		ErrMsgRes(0, strNoPh2plus);
		Exit(false);
	}
}

Boolean SendSmsMessage(Char *smsc, Char *phone, UInt8 flags, Char *text, UInt16 msgnum, UInt16 msgall)
{
	Char pdustr[512+1];
	Err err;
	if ((text != NULL) && (StrLen(text) > 0)) {
		UInt32 wt;
		UInt16 pdulen;
		Char cmdstr[500];
		if (msgall > 1)
			StrPrintF(cmdstr, "%i/%i", msgnum, msgall);
		PrgUpdateDialog(prgP, 0, strSendSMS, ((msgall>1)?cmdstr:NULL), true);
		pdulen = MakePduData(pdustr, smsc, phone, flags, text);
#ifdef DEBUG
		ErrMsg(0, "Send: %s, %s, %s = %s", smsc, phone, text, pdustr);
		return true;
#endif
		PrgCancelTest;
		StrPrintF(cmdstr, "AT+CMGS=%i\r", pdulen);

//		 ErrMsg(0, "Sending AT+CMGS=%i", pdulen);

		err = SerialSendString(portID, cmdstr);
		if (err) {
		    ErrMsg(0, "Failed AT+CMGS=%i", pdulen);
		    Exit(false);
		}

//experimental

		SrmReceiveFlush(portID, 100);

//		SetWaitTime(15);
//		PrgCancelTest;
//		ErrMsg(0, "Waiting for >");

		//err = SerialReceiveString(portID, rcvBuf, prgP);
		PrgCancelTest;
//		ATerr;
		// popitka ispravit'
		//rcvBuf[3] = 0;
		//if (StrNCompare(rcvBuf, "> ", 2) != 0) {
		//	ErrMsg(0, "Command +CMGS error:", rcvBuf);
		//	Exit(false);
		//}
//		PrgCancelTest;

//		ErrMsg(0, "Sending PDU=%s", pdustr);

		err = SerialSendString(portID, pdustr);
		if (err) {
		    ErrMsg(0, "Send PDU failed");
		    Exit(false);
		}
		
		PrgCancelTest;
		
		SrmReceiveFlush(portID, 100);//skinut echo PDU hex
		
		PrgCancelTest;

		err = SerialSendString(portID, "\032");
		if (err) {
		    ErrMsg(0, "Send ctl-z failed");
		    Exit(false);
		}
		
		PrgUpdateDialog(prgP, 0, strWaitResp, NULL, true);

		wt = SetWaitTime(0); SetWaitTime(wt * 4);

		err = SerialReceiveString(portID, rcvBuf, prgP);

		PrgCancelTest;
		ATerr;

		SetWaitTime(wt);
	}
	return true;
}

Boolean SendSmsEnd()
{
	Err err;
#ifdef DEBUG
	PrgUpdateDialog(prgP, 0, strDone, NULL, true);
	PrgStopDialog(prgP, true);
	return true;
#endif
	if (globalFlags.ModemSetup) {
		PrgUpdateDialog(prgP, 0, strRestModem, NULL, true);
		PrgCancelTest;

		if (saveCSMS != 1) {
			StrPrintF(rcvBuf, "AT+CSMS=%i\r", saveCSMS);
			if (SerialSendString(portID, rcvBuf)) Exit(false);
			err = SerialReceiveString(portID, rcvBuf, prgP);
			PrgCancelTest;
			ATerr;
		}
		
		if (saveCMGF != 0) {
			StrPrintF(rcvBuf, "AT+CMGF=%i\r", saveCMGF);
			if (SerialSendString(portID, rcvBuf)) Exit(false);
			err = SerialReceiveString(portID, rcvBuf, prgP);
			PrgCancelTest;
			ATerr;
		}

		if (saveCMEE != 2) {
			StrPrintF(rcvBuf, "AT+CMEE=%i\r", saveCMEE);
			if (SerialSendString(portID, rcvBuf)) Exit(false);
			err = SerialReceiveString(portID, rcvBuf, prgP);
			PrgCancelTest;
			ATerr;
		}
	}
	
	PrgUpdateDialog(prgP, 0, strDone, NULL, true);
	SerialClose(portID);
	PrgStopDialog(prgP, true);
	return true;
}
