// -*- C++ -*-
// File: Serial.c
//
// Created: Sun Jan  7 21:25:33 2001
//
// $Id: serSerial.c,v 1.1.1.1 2002/07/17 12:47:33 oskin Exp $
//
// SER
#include <stdarg.h>
#include <SerialMgrOld.h>
#include "../Serial.h"
#include "../../RuSMSRsc.h"

static UInt32 waitTime;
static DmOpenRef usmsRes = 0;
static Char *newRcvBuffer;

Boolean rcvDebug;

#define MaxCountLineErr 3
#define rcvBufSize 1024

void sErrMsg(Err err, Char *format, ...)
{
	va_list args;
	Char str[300], serr[100];
	va_start(args, format);
	if ((format == NULL) || (format[0] == chrNull))
		str[0] = chrNull;
	else
		StrVPrintF(str, format, args);
	if (err) {
		SysErrString(err, serr, 100);
		FrmCustomAlert(ErrorAlert, str, " Sys:", serr);
	} else
		FrmCustomAlert(ErrorAlert, str, "", "");
	va_end(args);
}
#define ErrMsg sErrMsg

Err SerialInit(void)
{
	Err err;
	UInt32 v;
	
	if ((usmsRes = DmOpenDatabaseByTypeCreator(
		'Rsrc', 'uSMS', dmModeReadOnly)) == 0)
		return DmGetLastErr();

	err = FtrGet(sysFtrCreator, sysFtrNumROMVersion, &v);
	if (err) return err;
	if (v < 0x03003000) {
		ErrMsg(0, "It is required PalmOS version 3.0 or more.");
		return errAlreadyShow;
	}
	return 0;
}

void SerialDone(void)
{
	if (usmsRes)
		DmCloseDatabase(usmsRes);
}

UInt32 SetWaitTime(UInt32 val)
{
        UInt32 old;

        old = waitTime;
        waitTime = val;
        return old;
}

UInt16 SerialOpen(ConnListType *conn)
{
	UInt16 ret, x;
	Err err;
	UInt32 baud, maxbaud;
	Char *p;
	SerSettingsType serSets;
	err = SysLibFind("Serial Library", &ret);
	if (err) {
		ErrMsg(err, "Serial Library not found.");
		return 0;
	}
	x = sizeof(UInt32);
	err = SerControl(ret, serCtlMaxBaud, &maxbaud, &x);
#ifdef ComLinkBug
	if (err == 0xffff) {
		err = 0;
		maxbaud = 57600;
	}
#endif
	if (err) {
		ErrMsg(err, "Setting serial");
		return 0;
	}
	p = conn->list[conn->sel];
	while((*p < chrDigitZero) && (*p > chrDigitNine)) p++;
	baud = (UInt32)StrAToI(p);
	if (baud == 0) {
		ErrMsg(0, "Incorrect speed.");
		return 0;
	} else if (baud > maxbaud) {
		ErrMsg(0, "This device is not supported %lu bps, set %lu bps.",
			   baud, maxbaud);
		baud = maxbaud;
	}
	err = SerOpen(ret, 0, baud);
	if (err) {
		ErrMsg(err, "Can't open serial");
		return 0;
	}
	SerGetSettings(ret, &serSets);
        serSets.flags &= 0xFFFFFF00;
        serSets.flags |= srmSettingsFlagBitsPerChar8 | srmSettingsFlagStopBits1;
	err = SerSetSettings(ret, &serSets);
	if (err) {
		ErrMsg(err, "Setting serial");
		return 0;
	}
    newRcvBuffer = MemPtrNew(rcvBufSize);
    if (newRcvBuffer) {
		err = SerSetReceiveBuffer(ret, (void*)newRcvBuffer, rcvBufSize);
		if (err) {
			ErrMsg(err, "Set Receive Buffer");
			return 0;
		}
	} else {
		ErrMsg(memErrNotEnoughSpace, "Set Receive Buffer");
		return memErrNotEnoughSpace;
	}
	waitTime = 15;
	return ret;
}

void SerialReceiveFlush(UInt16 thePort, UInt16 tics)
{
    SerReceiveFlush(thePort, tics);
}

Err SerialClose(UInt16 thePort)
{
    SerSetReceiveBuffer(thePort, 0L, 0);
    MemPtrFree(newRcvBuffer);
	
	return SerClose(thePort);
}

Err SerialReceiveString(UInt16 thePort, Char *rcvDataP, ProgressPtr prgP)
{
	UInt32 bytesLeft, bytesWait, startTime, maxRcvBlkSize, bytesRcvd;
	Char *rcvP;
	Err error;
	Char *rcvErr, *rcvOk, *r;
	EventType e;
	UInt8 countLineErr = MaxCountLineErr;

	rcvErr = rcvOk = NULL;
	
    maxRcvBlkSize = rcvBufSize;
    bytesLeft = rcvBufSize - 1;
	rcvP = rcvDataP;
    
	do {
		if (bytesLeft < maxRcvBlkSize)
			maxRcvBlkSize = bytesLeft; 
		startTime = TimGetSeconds();
		while (startTime+waitTime > TimGetSeconds()) {
			bytesWait = 0;
			SerReceiveCheck(thePort, &bytesWait);
			if (bytesWait)
				break;
			EvtGetEvent(&e, 1);
			PrgHandleEvent(prgP, &e);
			if (PrgUserCancel(prgP)) goto Exit;
		}
		if (bytesWait > 0) {
			bytesRcvd = SerReceive(thePort, rcvP, bytesWait, 1, &error);
			bytesLeft -= bytesRcvd;
			rcvP += bytesRcvd;
			*rcvP = '\0';
		} else
			error = serErrTimeOut;

		if ((error == serErrLineErr) && countLineErr) {
			Boolean ctsOn, dsrOn;
			SerGetStatus(thePort, &ctsOn, &dsrOn);
			SerClearErr(thePort);
			countLineErr--;
			continue;
		} else if (error) {
			ErrMsg(error, "");
			goto Exit;
		}
		countLineErr = MaxCountLineErr;
		/* ERROR, end of receive */
		if (((rcvErr = StrStr(rcvDataP, "\rERROR")) != NULL) ||
			((rcvErr = StrStr(rcvDataP, "\nERROR")) != NULL) ||
			((rcvErr = StrStr(rcvDataP, "\r+CME ERROR:")) != NULL) ||
			((rcvErr = StrStr(rcvDataP, "\n+CME ERROR:")) != NULL) ||
			((rcvErr = StrStr(rcvDataP, "\r+CMS ERROR:")) != NULL) ||
			((rcvErr = StrStr(rcvDataP, "\n+CMS ERROR:")) != NULL)) {
			if ((*(rcvP-1) != chrCarriageReturn) && (*(rcvP-1) != chrLineFeed))
				continue;

			rcvErr++;
			if (StrNCaselessCompare(rcvErr, "ERROR", 5) == 0)
				error = errModemError;
			else
				error = errModemErrMsg;
			r = rcvErr;
			while ((*r != chrCarriageReturn) && (*r != chrLineFeed))
				r++;
			*r = chrNull;
			bytesRcvd = StrLen(rcvErr);
			r = (Char*)MemPtrNew(sizeof(Char)*(bytesRcvd+1));
			StrCopy(r, rcvErr);
			*rcvDataP = chrNull;
			StrCopy(rcvDataP, r);
			MemPtrFree(r);
			goto Exit;
		}
		/* OK - End of receive */
		if (((rcvOk = StrStr(rcvDataP, "\rOK")) != NULL) ||
			((rcvOk = StrStr(rcvDataP, "\nOK")) != NULL)) {
			if ((*(rcvP-1) != chrCarriageReturn) && (*(rcvP-1) != chrLineFeed))
				continue;
			rcvOk++;
			*rcvOk = chrNull;
			rcvOk = rcvDataP;
			/* Delete echo of AT command */
			if ((StrNCaselessCompare(rcvOk, "AT", 2) == 0) ||
				((rcvOk = StrStr(rcvDataP,"\rAT")) != NULL) ||
				((rcvOk = StrStr(rcvDataP,"\nAT")) != NULL)) {
				if ((*rcvOk == chrCarriageReturn) ||
					(*rcvOk == chrLineFeed)) rcvOk++;
				while ((*rcvOk != chrCarriageReturn) &&
					   (*rcvOk != chrLineFeed)) rcvOk++;
				while ((*rcvOk == chrCarriageReturn) ||
					   (*rcvOk == chrLineFeed)) rcvOk++;
				bytesRcvd = StrLen(rcvOk);
				r = (Char*)MemPtrNew(sizeof(Char)*(bytesRcvd+1));
				StrCopy(r, rcvOk);
				*rcvDataP = chrNull;
				StrCopy(rcvDataP, r);
				MemPtrFree(r);
			}
			goto Exit;
		}
		/* Prompt for PDU data */
		r = rcvP;
		r -= 3;
		if ((StrNCompare(r, "\r> ", 3) == 0) ||
			(StrNCompare(r, "\n> ", 3) == 0)) {
			StrCopy(rcvDataP, "> ");
			goto Exit;
		}

    } while (bytesLeft);
    
  Exit:
    return error;
}

Err SerialSendString(UInt16 thePort, Char *data)
{	
	UInt32 v;
	Err err;
	v = SerSend(thePort, data, StrLen(data), &err);
	if (err == 0)
		err = SerSendWait(thePort, -1);
	if (err)
		ErrMsg(err, "Send");
	else
		err = 0;
	return err;;
}
