// -*- C++ -*-
// File: Serial.c
//
// Created: Sun Jan  7 21:26:04 2001
//
// $Id: srmSerial.c,v 1.1.1.1 2002/07/17 12:47:33 oskin Exp $
//
// SRM
#include <stdarg.h>
#include "../Serial.h"
#include "../../RuSMSRsc.h"

static UInt32 waitTime = 0;
static DmOpenRef usmsRes = NULL;
static Char *newRcvBuffer;

Boolean rcvDebug;

#define MaxCountLineErr 3
#define rcvBufSize 1024

void sErrMsg(Err err, Char *format, ...)
{
	va_list args;
	Char str[300], serr[100];
	va_start(args, format);
	if ((format == NULL) || (format[0] == chrNull))
		str[0] = chrNull;
	else
		StrVPrintF(str, format, args);
	if (err) {
		SysErrString(err, serr, 100);
		FrmCustomAlert(ErrorAlert, str, " Sys:", serr);
	} else
		FrmCustomAlert(ErrorAlert, str, "", "");
	va_end(args);
}
#define ErrMsg sErrMsg

Err SerialInit(void)
{
	Err err;
	UInt32 v;
	UInt32 romVersion;

	if ((usmsRes = DmOpenDatabaseByTypeCreator(
		'Rsrc', 'uSMS', dmModeReadOnly)) == 0)
		return DmGetLastErr();

	err = FtrGet(sysFileCSerialMgr, sysFtrNewSerialPresent, &v);
	if (err) return err;

	err = FtrGet(sysFtrCreator, sysFtrNumROMVersion, &romVersion);
	if (err) return err;

	if (v == 0) {
		ErrMsg(0, "New Serial Manager not installed: %i", v);
		return errAlreadyShow;
	}

//	if (romVersion >= 0x04003000) {
//		ErrMsg(0, "Old ROM: %u", romVersion);
//		return errAlreadyShow;
//	}
	return 0;
}

void SerialDone(void)
{
	if (usmsRes)
		DmCloseDatabase(usmsRes);
}

UInt32 SetWaitTime(UInt32 val)
{
	UInt32 old;

	old = waitTime;
	waitTime = val;
	return old;
}

UInt16 SerialOpen(ConnListType *conn)
{
	Char *Profile;
	UInt32 port, baud, flags = 0;
	UInt16 ret;
	UInt16 paramSize;
	Err err;
//	Int32 ctsTimeout = SysTicksPerSecond() / 2;

	if (conn->sel == (conn->cnt - 2)) /* -Current-? */
		Profile = NULL;
	else
		Profile = conn->list[conn->sel];
	err = CncGetProfileInfo(Profile, &port, &baud, NULL, NULL, NULL,
							NULL, NULL, NULL);
	if (err) {
		ErrMsg(err, "Cannot get connection info.");
		return 0;
	}
	
//	port = sysFileCVirtIrComm; //oleg test
//	baud = 9600;
//make
	err = SrmOpen(port, baud, &ret);
	if (err) {
		ErrMsg(err, "Cannot open communication port.");
		return 0;
	}

	//err = SrmControl(ret, srmCtlIrDAEnable, NULL, NULL);
	//err = SrmControl(ret, srmCtlRxEnable, NULL, NULL);

//	paramSize = sizeof(UInt32);
//	paramSize = sizeof(Int16 *);
//	err = SrmControl(ret, srmCtlSetFlags, &flags, &paramSize);
//	err = SrmControl(ret, srmCtlGetFlags, &flags, &paramSize);
//	if (err != errNone) {
//		ErrMsg(err, "SrmControl");
//		return 0;
//	}
//	paramSize = sizeof(UInt32);
	paramSize = sizeof(UInt32);
//	flags &= 0xFFFFFF00;
	flags = srmSettingsFlagRTSAutoM  | srmSettingsFlagCTSAutoM;
	flags |= srmSettingsFlagBitsPerChar8;// 
	flags |= (srmSettingsFlagStopBitsM | srmSettingsFlagStopBits1);
//	flags |= srmSettingsFlagParityOnM | srmSettingsFlagParityEvenM;
//	flags |= (srmSettingsFlagStopBitsM | srmSettingsFlagStopBits1);
	err = SrmControl(ret, srmCtlSetFlags, &flags, &paramSize);

#ifdef ComLinkBug
	if (err == 0xffff) {
		err = 0;
	}
#endif
	if (err) {
		ErrMsg(err, "Setting serial");
		return 0;
	}

//	SrmReceiveFlush(ret, 0);

//	paramSize = sizeof(UInt32);
//	err = SrmcControl(ret, srmCtlSetFlags, &flags, &paramSize);

//	paramSize = sizeof(Int32);
//	err = SrmControl(ret, srmCtlSetCtsTimeout, &ctsTimeout, &paramSize);

//	if (err) {
//		ErrMsg(err, "Setting CTS Timeout");
//		return 0;
//	}

    newRcvBuffer = MemPtrNew(rcvBufSize);
    if (newRcvBuffer) {
		err = SrmSetReceiveBuffer(ret, (void*)newRcvBuffer, rcvBufSize);
		if (err) {
			ErrMsg(err, "Set Receive Buffer");
			return 0;
		}
	} else {
		ErrMsg(memErrNotEnoughSpace, "Set Receive Buffer");
		return memErrNotEnoughSpace;
	}
	waitTime = 15;
	return ret;
}

void SerialReceiveFlush(UInt16 thePort, UInt16 tics)
{
    SrmReceiveFlush(thePort, tics);
}

Err SerialClose(UInt16 thePort)
{
    SrmSetReceiveBuffer(thePort, NULL, NULL);
    MemPtrFree(newRcvBuffer);
	
	return SrmClose(thePort);
}

Err SerialReceiveString(UInt16 thePort, Char *rcvDataP, ProgressPtr prgP)
{
	UInt32 bytesLeft, bytesWait, startTime, maxRcvBlkSize, bytesRcvd;
	Char *rcvP;
	Err error;
	Char *rcvErr, *rcvOk, *r;
	EventType e;
	UInt8 countLineErr = MaxCountLineErr;
	Int32 ticks = SysTicksPerSecond() / 2;

	rcvErr = rcvOk = NULL;
	
    maxRcvBlkSize = rcvBufSize;
    bytesLeft = rcvBufSize - 1;
	rcvP = rcvDataP;
    
	do {
		if (bytesLeft < maxRcvBlkSize)
			maxRcvBlkSize = bytesLeft;
		startTime = TimGetSeconds();
		while (startTime+waitTime > TimGetSeconds()) {
//experimental
			if(EvtEventAvail()) {
			EvtGetEvent(&e, 10);
			PrgHandleEvent(prgP, &e);
			}
			if (PrgUserCancel(prgP)) goto Exit;
			bytesWait = 0;

//			error = SrmReceiveWait(thePort, 1, 100);
//			if(error != errNone) {
//			ErrMsg(error, "SrmReceiveWait %i", bytesWait);
//			}

			error = SrmReceiveCheck(thePort, &bytesWait);
			if(error != errNone) {
//experimental
			SrmClearErr(thePort);
//			ErrMsg(error, "SrmReceiveCheck %i", bytesWait);
			}

			if (bytesWait)
				break;
		}
		if (bytesWait > 0) {
			bytesRcvd = SrmReceive(thePort, rcvP, bytesWait, ticks, &error);
			bytesLeft -= bytesRcvd;
			rcvP += bytesRcvd;
			*rcvP = 0;
		} else {
			if (error == errNone) error = serErrTimeOut;
			ErrMsg(error, "Timeout in Receive");
			goto Exit;
		}

		if ((error == serErrLineErr) && countLineErr) {
			UInt32 statusField;
			UInt16 lineErrs;
			SrmGetStatus(thePort, &statusField, &lineErrs);
			SrmClearErr(thePort);
			countLineErr--;
			continue;
		} else if (error != errNone) {
			ErrMsg(error, "");
			goto Exit;
		}
		countLineErr = MaxCountLineErr;
		/* ERROR, end of receive */
		if (((rcvErr = StrStr(rcvDataP, "\rERROR")) != NULL) ||
			((rcvErr = StrStr(rcvDataP, "\nERROR")) != NULL) ||
			((rcvErr = StrStr(rcvDataP, "\r+CME ERROR:")) != NULL) ||
			((rcvErr = StrStr(rcvDataP, "\n+CME ERROR:")) != NULL) ||
			((rcvErr = StrStr(rcvDataP, "\r+CMS ERROR:")) != NULL) ||
			((rcvErr = StrStr(rcvDataP, "\n+CMS ERROR:")) != NULL)) {
			if ((*(rcvP-1) != chrCarriageReturn) && (*(rcvP-1) != chrLineFeed))
				continue;

			rcvErr++;
			if (StrNCaselessCompare(rcvErr, "ERROR", 5) == 0)
				error = errModemError;
			else
				error = errModemErrMsg;
			r = rcvErr;
			while ((*r != chrCarriageReturn) && (*r != chrLineFeed))
				r++;
			*r = chrNull;
			bytesRcvd = StrLen(rcvErr);
			r = (Char*)MemPtrNew(sizeof(Char)*(bytesRcvd+1));
			StrCopy(r, rcvErr);
			*rcvDataP = chrNull;
			StrCopy(rcvDataP, r);
			MemPtrFree(r);
			goto Exit;
		}
		/* OK - End of receive */
		if (((rcvOk = StrStr(rcvDataP, "\rOK")) != NULL) ||
			((rcvOk = StrStr(rcvDataP, "\nOK")) != NULL)) {
			if ((*(rcvP-1) != chrCarriageReturn) && (*(rcvP-1) != chrLineFeed))
				continue;
			rcvOk++;
			*rcvOk = chrNull;
			rcvOk = rcvDataP;
			/* Delete echo of AT command */
			if ((StrNCaselessCompare(rcvOk, "AT", 2) == 0) ||
				((rcvOk = StrStr(rcvDataP,"\rAT")) != NULL) ||
				((rcvOk = StrStr(rcvDataP,"\nAT")) != NULL)) {
				if ((*rcvOk == chrCarriageReturn) ||
					(*rcvOk == chrLineFeed)) rcvOk++;
				while ((*rcvOk != chrCarriageReturn) &&
					   (*rcvOk != chrLineFeed)) rcvOk++;
				while ((*rcvOk == chrCarriageReturn) ||
					   (*rcvOk == chrLineFeed)) rcvOk++;
				bytesRcvd = StrLen(rcvOk);
				r = (Char*)MemPtrNew(sizeof(Char)*(bytesRcvd+1));
				StrCopy(r, rcvOk);
				*rcvDataP = chrNull;
				StrCopy(rcvDataP, r);
				MemPtrFree(r);
			}
			goto Exit;
		}
		/* Prompt for PDU data */
		r = rcvP;
		r -= 3;
		//izmenilas' v 4.0 oleg
		//r[3] = 0x0;
		if ((StrNCompare(r, "\r> ", 3) == 0) ||
			(StrNCompare(r, "\n> ", 3) == 0)) {
			StrCopy(rcvDataP, "> ");
			goto Exit;
		}

    } while (bytesLeft);
    
  Exit:
    return error;
}

Err SerialSendString(UInt16 thePort, Char *data)
{
	UInt32 v;
	Err err;
	EventType e;

	//err = SrmControl(thePort, srmCtlRxDisable, NULL, NULL);

	v = SrmSend(thePort, data, StrLen(data), &err);
	if (err == errNone)
	err = SrmSendWait(thePort);
	if (err != errNone)
		ErrMsg(err, "Send");
//	else
//		err = errNone;

	//err = SrmControl(thePort, srmCtlRxEnable, NULL, NULL);

//experimental
	if(EvtEventAvail()) {
	EvtGetEvent(&e, 10);
	SysHandleEvent(&e);
	}

	return err;
}
