// -*- C++ -*-
// File: utils.c
//
// Created: Mon Jul 24 19:41:32 2000
//
// $Id: utils.c,v 1.1.1.1 2002/07/17 12:47:33 oskin Exp $
//

#include <PalmOS.h>
#include <stdarg.h>
#include "RuSMS.h"

FieldPtr SetFieldTextFromHandle(UInt16 fieldID, MemHandle txtH)
{
    MemHandle      oldTxtH;
    FormPtr        frm = FrmGetActiveForm();
    FieldPtr       fldP;

    fldP = FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, fieldID));
    ErrNonFatalDisplayIf(!fldP, "missing field");
    oldTxtH = FldGetTextHandle(fldP);
    FldSetTextHandle(fldP, txtH);
    FldDrawField(fldP);
    if(oldTxtH) MemHandleFree(oldTxtH);
    return fldP;
}

FieldPtr SetFieldTextFromStr(UInt16 fieldID, Char* strP)
{
    MemHandle txtH;

    txtH = MemHandleNew(StrLen(strP) + 1);
    if(!txtH) return NULL;
    StrCopy(MemHandleLock(txtH), strP);
    MemHandleUnlock(txtH);
    return SetFieldTextFromHandle(fieldID, txtH);
} 

FieldPtr GetFocusedField(void)
{
    FormPtr form;
    UInt16 focus;
    FormObjectKind objType;

    form = FrmGetActiveForm();
    focus = FrmGetFocus(form);
    if (focus == noFocus)
        return NULL;

    objType = FrmGetObjectType(form, focus);
    if (objType == frmFieldObj)
        return FrmGetObjectPtr(form, focus);
    else if (objType == frmTableObj)
        return TblGetCurrentField(FrmGetObjectPtr(form, focus));
    else
        return NULL;
}

Int16 StringLenInPixel(Char *text, FontID font)
{
	FontID curfont;
	Int16 ret;

	curfont = FntSetFont(font);
	ret = FntCharsWidth(text, StrLen(text));
	FntSetFont(curfont);
	return ret;
}

Int16 MaxStringLenInPixel(Char **list, UInt16 cnt, FontID font)
{
	FontID curfont;
	UInt16 i;
	Int16 ret, w;

	curfont = FntSetFont(font);
	ret = 0;
	for (i = 0; i < cnt; i++) {
		w = FntCharsWidth(list[i], StrLen(list[i]));
		if (w > ret) ret = w;
	}
	FntSetFont(curfont);
	return ret;
}

UInt16 SystemError(Err err)
{
    UInt16 ret;
    Char *str;

    str = MemPtrNew(100);
    ret = FrmCustomAlert(FatalErrorAlert, SysErrString(err, str, 100), "", "");
    MemPtrFree(str);
    return ret;
}

Int16 SmscGetIndex(SmscListType list, Char *smsc)
{
    Int16 i;
    for (i = 0; i < list.cnt-1; i++)
	if (StrCompare(list.name[i], smsc) == 0)
	    return i;
    return -1;
}

typedef struct {
    Char *name;
    Char *phone;
	SmscFlagsType flags;
} SmscSortType;

static Int16 SmscSortF (SmscSortType *c1, SmscSortType *c2, Int32 x)
{
    return StrCaselessCompare (c1->name, c2->name);
}

void SmscSortList(SmscListType list)
{
    SmscSortType *tmp;
    UInt16 i;

    tmp = (SmscSortType*)MemPtrNew(sizeof(SmscSortType)*(list.cnt-2));
    for (i = 1; i < list.cnt-1; i++) {
		tmp[i-1].name = list.name[i];
		tmp[i-1].phone = list.phone[i];
		tmp[i-1].flags = list.flags[i];
    }
    SysQSort(tmp, list.cnt-2, sizeof(SmscSortType), (CmpFuncPtr)SmscSortF, 0);
    for (i = 1; i < list.cnt-1; i++) {
		list.name[i] = tmp[i-1].name;
		list.phone[i] = tmp[i-1].phone;
		list.flags[i] = tmp[i-1].flags;
    }
    MemPtrFree(tmp);
}

Boolean ValidCharPhone(Char c)
{
	switch(c) {
	case chrDigitZero:
	case chrDigitOne:
	case chrDigitTwo:
	case chrDigitThree:
	case chrDigitFour:
	case chrDigitFive:
	case chrDigitSix:
	case chrDigitSeven:
	case chrDigitEight:
	case chrDigitNine:
	case chrPlusSign:
	case chrHyphenMinus:
	case chrLeftParenthesis:
	case chrRightParenthesis:
	case chrSpace:
	    break;
	default:
	    return false;
	}
    return true;
}

Boolean ValidPhone(Char *str)
{
    while (*str != chrNull)
		if (!ValidCharPhone(*str++))
			return false;
	return true;
}

Char *PhoneToDigits(Char *str)
{
    Char *ret, *s;
    ret = s = (Char*)MemPtrNew(StrLen(str)+1);
	if (*str == chrPlusSign)
		*s++ = *str++;
    while (*str != chrNull)
	switch(*str) {
	case chrDigitZero:
	case chrDigitOne:
	case chrDigitTwo:
	case chrDigitThree:
	case chrDigitFour:
	case chrDigitFive:
	case chrDigitSix:
	case chrDigitSeven:
	case chrDigitEight:
	case chrDigitNine:
	    *s++ = *str++;
	    break;
	default:
	    str++;
	    break;
	}
    *s++ = chrNull;
    return ret;
}

UInt16 MakePhoneList(Char *phone, PhoneType **Phones)
{
	UInt16 i, l, cnt, s, e;
	Char *p;

	if((phone == NULL) || (phone[0] == chrNull)) {
		*Phones = NULL;
		return 0;
	}
	l = StrLen(phone);
	for (i=0, cnt=1; i<l; i++)
		if ((phone[i] == chrSemicolon) || (phone[i] == chrLineFeed))
			cnt++;
	*Phones = (PhoneType*)MemPtrNew(sizeof(PhoneType)*cnt);
	for (l=0, s=0; l<cnt; l++) {
		(*Phones)[l].empty = 0;
		(*Phones)[l].bad = 0;
		e = s;
		while ((phone[e] != chrSemicolon) && (phone[e] != chrLineFeed) && (phone[e] != chrNull))
			e++;
		(*Phones)[l].num = (Char*)MemPtrNew(e-s+1);
		while ((e > s) && (IsSpace(GetCharAttr(), phone[s])))
			s++;
		if (e == s) {
			MemPtrFree((*Phones)[l].num);
			(*Phones)[l].num = NULL;
			(*Phones)[l].empty = 1;
			s++;
			continue;
		}
		p = (*Phones)[l].num;
		if (phone[s] == chrPlusSign)
			*p++ = phone[s++];
		while (e > s)
			switch (phone[s]) {
			case chrDigitZero:
			case chrDigitOne:
			case chrDigitTwo:
			case chrDigitThree:
			case chrDigitFour:
			case chrDigitFive:
			case chrDigitSix:
			case chrDigitSeven:
			case chrDigitEight:
			case chrDigitNine:
				*p++ = phone[s++];
				break;
			case chrHyphenMinus:
			case chrLeftParenthesis:
			case chrRightParenthesis:
			case chrSpace:
				s++;
				break;
			default:
				*p++ = phone[s++];
				Phones[l]->bad = 1;
				break;
			}
		*p = chrNull;
		s++;
	}
	return cnt;
}

void FreePhoneList(PhoneType **Phones, UInt16 cnt)
{
	if (*Phones == NULL) return;
	for (cnt--; cnt <= 0; cnt--)
		if ((*Phones)[cnt].num != NULL)
			MemPtrFree((*Phones)[cnt].num);
	MemPtrFree(*Phones);
	*Phones = NULL;
}

Char str[500], serr[100];

void ErrMsg(Err err, Char *format, ...)
{
	va_list args;
	va_start(args, format);
	if ((format == NULL) || (format[0] == chrNull))
		str[0] = chrNull;
	else
		StrVPrintF(str, format, args);
	if (err) {
		SysErrString(err, serr, 100);
		FrmCustomAlert(ErrorAlert, str, " Sys:", serr);
	} else
		FrmCustomAlert(ErrorAlert, str, "", "");
	va_end(args);
}

void ErrMsgRes(Err err, DmResID format, ...)
{
	va_list args;
	MemHandle resH;
	MemPtr resP;

	va_start(args, format);
	resH = DmGetResource('tSTR', format);
	resP = MemHandleLock(resH);
	StrVPrintF(str, resP, args);
	MemPtrUnlock(resP);
	DmReleaseResource(resH);
	if (err) {
		SysErrString(err, serr, 100);
		FrmCustomAlert(ErrorAlert, str, " Sys:", serr);
	} else
		FrmCustomAlert(ErrorAlert, str, "", "");
	va_end(args);
}

void WarnMsgRes(DmResID msg)
{
	MemHandle resH;
	MemPtr resP;

	resH = DmGetResource('tSTR', msg);
	resP = MemHandleLock(resH);
	StrCopy(str, resP);
	MemPtrUnlock(resP);
	DmReleaseResource(resH);
	FrmCustomAlert(WarnAlert, str, "", "");
}
